// IMPORTANT: Adafruit_TFTLCD LIBRARY MUST BE SPECIFICALLY
// CONFIGURED FOR EITHER THE TFT SHIELD OR THE BREAKOUT BOARD.
// SEE RELEVANT COMMENTS IN Adafruit_TFTLCD.h FOR SETUP.
//Test for TFT LCD Shield 2.8"
//by Open-Smart Team and Catalex Team
//catalex_inc@163.com
//Store:   http://dx.com
//Test Function: Display temperature value from LM75A sensor, and show touch results.
// Use Platformio.ini.
// Board: Arduino UNO R3 - use 3V3 setting - OpenSmart version.

#include <Adafruit_GFX.h>    // Core graphics library

#include <Wire.h>
#include <LM75.h>

// Touch screen library with X Y and Z (pressure) readings as well
#include <stdint.h>
#include "TouchScreen.h"

// These are the pins for the shield!
#define YP A1  // must be an analog pin, use "An" notation!
#define XM A2  // must be an analog pin, use "An" notation!
#define YM 7   // can be a digital pin
#define XP 6   // can be a digital pin

#define MINPRESSURE 10
#define MAXPRESSURE 1000

// For better pressure precision, we need to know the resistance
// between X+ and X- Use any multimeter to read it
// For the one we're using, its 300 ohms across the X plate
TouchScreen ts = TouchScreen(XP, YP, XM, YM, 397);
TSPoint tp;

#include <MCUFRIEND_kbv.h>
//----------------------------------------|
// TFT Breakout  -- Arduino UNO / Mega2560 / OPEN-SMART UNO Black
// GND              -- GND
// 3V3               -- 3.3V
// CS                 -- A3
// RS                 -- A2
// WR                -- A1
// RD                 -- A0
// RST                -- RESET
// LED                -- GND
// DB0                -- 8
// DB1                -- 9
// DB2                -- 10
// DB3                -- 11
// DB4                -- 4
// DB5                -- 13
// DB6                -- 6
// DB7                -- 7

MCUFRIEND_kbv tft;
LM75 sensor;  // initialize an LM75 object


unsigned long updatetime;


// Assign human-readable names to some common 16-bit color values:
#define	BLACK   0x0000
#define	BLUE    0x001F
#define	RED     0xF800
#define	GREEN   0x07E0
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE   0xFFFF

int16_t TS_LEFT = 0, TS_RT = 10000, TS_TOP = 0, TS_BOT = 10000;

void setup(void) {
    Serial.begin(9600);
    Wire.begin();
    Serial.println(F("TFT LCD test"));

//     #ifdef USE_ADAFRUIT_SHIELD_PINOUT
//     Serial.println(F("Using Adafruit 3.2\" TFT Arduino Shield Pinout"));
//     #else
//     Serial.println(F("Using Adafruit 3.2\" TFT Breakout Board Pinout"));
//     #endif

    Serial.print("TFT size is "); Serial.print(tft.width()); Serial.print("x"); Serial.println(tft.height());

    tft.reset();

    uint16_t identifier = tft.readID();

//     if(identifier == 0x9325) {
//         Serial.println(F("Found ILI9325 LCD driver"));
//     } else if(identifier == 0x9328) {
//         Serial.println(F("Found ILI9328 LCD driver"));
//     } else if(identifier == 0x7575) {
//         Serial.println(F("Found HX8347G LCD driver"));
//     } else if(identifier == 0x9341) {
//         Serial.println(F("Found ILI9341 LCD driver"));
//     } else if(identifier == 0x8357) {
//         Serial.println(F("Found HX8357D LCD driver"));
//     } else if(identifier == 0x4747) {
//         Serial.println(F("Found HX8347A LCD driver"));
//     } else if(identifier == 0x9326) {
//         Serial.println(F("Found ILI9326 LCD driver"));
//     } else if(identifier == 0x9327) {
//         Serial.println(F("Found ILI9327 LCD driver"));
//     } else 
       if(identifier == 0x9320) {
        Serial.println(F("Found ILI9320 LCD driver"));
    } else {
        Serial.print(F("Unknown LCD driver chip: "));
        Serial.println(identifier, HEX);
//         Serial.println(F("If using the Adafruit 2.8\" TFT Arduino shield, the line:"));
//         Serial.println(F("  #define USE_ADAFRUIT_SHIELD_PINOUT"));
//         Serial.println(F("should appear in the library header (Adafruit_TFT.h)."));
//         Serial.println(F("If using the breakout board, it should NOT be #defined!"));
//         Serial.println(F("Also if using the breakout, double-check that all wiring"));
//         Serial.println(F("matches the tutorial."));
        return;
    }

    tft.begin(identifier);
  
  
    for(uint8_t rotation=0; rotation<4; rotation++) {
        tft.fillScreen(BLACK);
        tft.setRotation(rotation);
        tft.setCursor(20, 20);
        tft.setTextColor(WHITE);  tft.setTextSize(1);
        tft.print(rotation);
        delay(1000);
    }
  
    tft.fillScreen(GREEN);
    tft.setRotation(0);

    updatetime = 5000UL + millis();
}

void loop(void) {
    
    Serial.println("Loop start");
    
    if((updatetime + 3000UL) < millis()){
        updatetime = millis();

        tft.setCursor(10, 110);
        tft.fillRect(8, 110, 146, 24, BLACK);
        tft.setTextColor(WHITE);  tft.setTextSize(3);
        tft.print(sensor.temp());
        tft.println(" C");
        
        Serial.print("Temp: "); Serial.print(sensor.temp()); Serial.println(" C");
    }
    
    // A point object holds x y and z coordinates.
    // VERY IMPORTANT: After every getPoint call, 
    // you have to set the 4 pins back to inputs again, 
    // otherwise the writing to the tft display fails!!!!!
    tp = ts.getPoint();
    pinMode(XM, OUTPUT);
    pinMode(YP, OUTPUT);
    pinMode(XP, OUTPUT);
    pinMode(YM, OUTPUT);

    // we have some minimum pressure we consider 'valid'
    // pressure of 0 means no pressing!
    if (tp.z > MINPRESSURE && tp.z < MAXPRESSURE) {
        
        tft.setCursor(10, 10);
        tft.fillRect(8, 10, 146, 24, BLACK);
        tft.setTextColor(WHITE);  tft.setTextSize(3);
        tft.print("X = "); tft.println(tp.x);
        tft.setCursor(10, 35);
        tft.fillRect(8, 35, 146, 24, BLACK);
        tft.setTextColor(WHITE);  tft.setTextSize(3);
        tft.print("Y = "); tft.println(tp.y);
        tft.setCursor(10, 60);
        tft.fillRect(8, 60, 146, 24, BLACK);
        tft.setTextColor(WHITE);  tft.setTextSize(3);
        tft.print("P = "); tft.println(tp.z);

        Serial.print("X = "); Serial.print(tp.x);
        Serial.print("\tY = "); Serial.print(tp.y);
        Serial.print("\tPressure = "); Serial.println(tp.z);
        
        // Determine the borders of the screen:
        TS_LEFT = (TS_LEFT < tp.x) ? tp.x : TS_LEFT;
        TS_RT   = (TS_RT > tp.x)   ? tp.x : TS_RT;
        TS_TOP  = (TS_TOP < tp.y)  ? tp.y : TS_TOP;
        TS_BOT  = (TS_BOT > tp.y)  ? tp.y : TS_BOT;

        tft.setTextColor(WHITE);  tft.setTextSize(2);
        tft.setCursor(10, 160);
        tft.fillRect(8, 160, 146, 17, BLACK);
        tft.print("Left: "); tft.println(TS_LEFT);
        tft.setCursor(10, 178);
        tft.fillRect(8, 178, 146, 17, BLACK);
        tft.print("Right: "); tft.println(TS_RT);
        tft.setCursor(10, 196);
        tft.fillRect(8, 196, 146, 17, BLACK);
        tft.print("Top: "); tft.println(TS_TOP);
        tft.setCursor(10, 213);
        tft.fillRect(8, 213, 146, 17, BLACK);
        tft.print("Bottom: "); tft.println(TS_BOT);
        
    }
}

